#include <processor_include.h>
#include <signal.h>
#include <string.h>
#include <filter.h>

#include "framework.h"

static float mic1[DSP_BLOCK_SIZE];
static float mic2[DSP_BLOCK_SIZE];
static float mic3[DSP_BLOCK_SIZE];
static float mic4[DSP_BLOCK_SIZE];

void process(int sig)
{
    int n;
 
    sample_t *u30 = dsp_get_audio_u30();    /* line 2 in without mic bias, no out */
    sample_t *u31 = dsp_get_audio_u31();    /* line 1 in with mic bias, no out */
    sample_t *u32 = dsp_get_audio_u32();    /* mic 1 and 2 in, headset out */
    sample_t *u33 = dsp_get_audio_u33();    /* mic 3 and 4 in, no out */

    for(n=0; n<DSP_BLOCK_SIZE; ++n) {
        mic1[n] = u32[n].right;
        mic2[n] = u32[n].left;
        mic3[n] = u33[n].right;
        mic4[n] = u33[n].left;
    }
    
    for(n=0; n<DSP_BLOCK_SIZE; ++n) {
        u32[n].right = mic1[n] + mic2[n];
        u32[n].left  = mic3[n] + mic4[n];
    }
}

static void keyboard(int sig)
{
    unsigned int keys = dsp_get_keys();
    unsigned int leds = 0;

    if(keys & DSP_SW1) {
        leds = 0;
    } else if(keys & DSP_SW2) {
        leds = DSP_D1;
    } else if(keys & DSP_SW3) {
        leds = DSP_D2;
    } else if(keys & DSP_SW4) {
        leds = DSP_D1|DSP_D2;
    }
    
    dsp_set_leds(leds);
}

static void timer(int sig)
{
}

void main()
{   
    /*
    Setup the DSP framework.
    */
    dsp_init();
    dsp_set_leds(DSP_D1 | DSP_D2);
 
    /*
    Register interrupt handlers:
    SIG_SP1: the audio callback
    SIG_USR0: the keyboard callback
    SIG_TMZ: the timer callback
    */
    interrupt(SIG_SP1, process);
    interrupt(SIG_USR0, keyboard);
    interrupt(SIG_TMZ, timer);
    timer_set(9830400, 9830400);
    /* timer_on(); */

    /*
    Start the framework.
    */
    dsp_start();
    
    /*
    Everything is handled by the interrupt handlers, so just put an empty
    idle-loop here. If not, the program falls back to an equivalent idle-loop
    in the run-time library when main() returns.
    */
    for(;;) {
        idle();
    }
}

